/*
 * cat1541-III.c
 * Copyright (c) 2007 Hannu Nuotio <hannu.nuotio@pp.inet.fi>
 * Written by Hannu Nuotio <hannu.nuotio@pp.inet.fi>
 */

/*
 * cat1541-III prints the output from 1541-III.
 *
 * usage:
 *  cat1541-III [-hv] [-t device]
 *	-h		help
 *	-v		verbose
 *	-t device	serial device (default = /dev/ttyS0)
 */

#include <sys/types.h>
#include <sys/stat.h>
#include <string.h>
#include <fcntl.h>
#include <termios.h>
#include <stdio.h>
#include <unistd.h>

void usage()
{
fprintf(stderr,"cat1541-III v.1.0\n");
fprintf(stderr,"-----------------\n");
fprintf(stderr," fwupload [-hv] [-t device]\n\n");
fprintf(stderr,"Prints output from 1541-III.\n\n");
fprintf(stderr," -h		this help text\n");
fprintf(stderr," -v		verbose\n");
fprintf(stderr," -t device	serial device (default = /dev/ttyS0)\n");
}

int main(int argc, char *argv[])
{
int com, c, res, vflg=0;
struct termios oldtio, newtio;
unsigned char x;

char *comdevice = "/dev/ttyS0";
extern char *optarg;
extern int optind, optopt;

while((c = getopt(argc,argv,"hvt:")) != -1) {
	switch(c) {
	case 'h':
		usage();
		return 0;
		break;	
	case 'v':
		vflg=1;
		break;
	case 't':
		comdevice=optarg;
		break;
	case ':':
		fprintf(stderr,"-%c requires an operand\n",c);
		return 1;
		break;
	case '?':
		fprintf(stderr,"-%c is unknown\n",c);
		return 1;
		break;
	}
}

if (vflg) {fprintf(stderr,"Opening %s...",comdevice);}

com = open(comdevice, O_RDONLY | O_NOCTTY ); 
if (com < 0) {perror(comdevice); return 1; }

if (vflg) {fprintf(stdout,"ok.\n");}
        
tcgetattr(com,&oldtio);
        
memset(&newtio, 0, sizeof(newtio));
newtio.c_cflag = B115200 | CS8 | CLOCAL | CREAD;
newtio.c_iflag = IGNPAR;
newtio.c_oflag = 0;
        
newtio.c_lflag = 0;
         
newtio.c_cc[VTIME] = 0;
newtio.c_cc[VMIN]  = 1;

tcflush(com, TCIFLUSH);
tcsetattr(com,TCSANOW,&newtio);

if (vflg) {fprintf(stderr,"Showing 1541-III output (^C to stop):\n");}
while((res = read(com,&x,1))==1) {printf("%c",x); fflush(stdout);}

if (vflg) {fprintf(stderr,"Cleaning up...");}
tcsetattr(com,TCSANOW,&oldtio);
close(com);
if (vflg) {fprintf(stderr,"done.\n");}

return 0;
}
